import * as GameMenu from './game_menu.js';
import * as Options from '../storage/options.js';
import * as HighScore from '../storage/high_score.js';
import * as MainMenu from '../menu/main_menu.js';
import Snake from './snake.js';
import Wall from './wall.js';
import Timer from '../other/timer.js';
import Interval from '../other/interval.js';
import PopupWindow from "../other/popup_window.js";
import Timeout from "../other/timeout.js";
import { Direction, showHideCanvas } from '../main.js';
import { EVENT_KEY } from '../other/utilities.js';
import { Grid, ItemType } from "./grid.js";
import { setupWalls } from './maps.js';
import { Apple, Orange, Banana } from './all_foods.js';
const TIMEOUTS = [];
const INTERVALS = [];
const COLLISIONS = [];
let TIMER;
let TWO_PLAYER_MODE = false;
let MAP_NAME;
let GAME_OVER = false;
let PAUSE = false;
let STAGE;
let GRID;
const SNAKES = [];
const SNAKES_COLORS = [
    'green',
    'dodgerblue'
];
window.onkeydown = function (event) {
    var returnValue;
    for (var i = 0; i < SNAKES.length; i++) {
        returnValue = SNAKES[i].onKeyDown(event.keyCode);
        if (!returnValue) {
            return returnValue;
        }
    }
    return true;
};
window.onkeyup = function (event) {
    var returnValue;
    for (var i = 0; i < SNAKES.length; i++) {
        returnValue = SNAKES[i].onKeyUp(event.keyCode);
        if (!returnValue) {
            return returnValue;
        }
    }
    return true;
};
export function init(canvas) {
    TIMER = new Timer(GameMenu.updateTimer);
    createjs.Ticker.on('tick', tick);
    STAGE = new createjs.Stage(canvas);
}
export function start(mapName, twoPlayersMode) {
    if (typeof twoPlayersMode == 'undefined') {
        twoPlayersMode = false;
    }
    GAME_OVER = false;
    TWO_PLAYER_MODE = twoPlayersMode;
    MAP_NAME = mapName;
    TIMER.restart();
    GameMenu.updateTimer(TIMER.getString());
    const columns = Options.get('columns');
    const lines = Options.get('lines');
    GRID = new Grid({
        columns: columns,
        lines: lines,
        onCollision: collisionOccurred
    });
    setupSnakes(twoPlayersMode);
    setupFrame();
    const interval = setupWalls(mapName, SNAKES, GRID);
    if (interval) {
        INTERVALS.push(interval);
    }
    setupAppleInterval();
    setupOrangeInterval();
    setupBananaInterval();
    setupSnakeMovement();
    for (let a = 0; a < SNAKES.length; a++) {
        const snake = SNAKES[a];
        updateScore(snake);
    }
    showHideCanvas(true);
    GameMenu.show(TWO_PLAYER_MODE);
}
function setupSnakes(twoPlayersMode) {
    const lines = GRID.args.lines;
    const columns = GRID.args.columns;
    const snakeSpeed = Options.get('snakeSpeed');
    const midLine = Math.round(lines / 2);
    const leftColumn = Math.round(columns * 0.1);
    const rightColumn = Math.round(columns * 0.9);
    if (twoPlayersMode) {
        const position1 = 0;
        const snake1 = new Snake({
            speed: snakeSpeed,
            position: {
                column: leftColumn,
                line: midLine
            },
            startingDirection: Direction.east,
            color: SNAKES_COLORS[position1],
            keyboardMapping: {
                left: EVENT_KEY.a,
                right: EVENT_KEY.d,
                up: EVENT_KEY.w,
                down: EVENT_KEY.s
            }
        });
        const position2 = 1;
        const snake2 = new Snake({
            speed: snakeSpeed,
            position: {
                column: rightColumn,
                line: midLine
            },
            startingDirection: Direction.west,
            color: SNAKES_COLORS[position2],
            keyboardMapping: {
                left: EVENT_KEY.leftArrow,
                right: EVENT_KEY.rightArrow,
                up: EVENT_KEY.upArrow,
                down: EVENT_KEY.downArrow
            }
        });
        SNAKES.push(snake1);
        SNAKES.push(snake2);
    }
    else {
        const position = 0;
        const snake = new Snake({
            speed: snakeSpeed,
            position: {
                column: leftColumn,
                line: midLine
            },
            startingDirection: Direction.east,
            color: SNAKES_COLORS[position],
            keyboardMapping: {
                left: EVENT_KEY.a,
                left2: EVENT_KEY.leftArrow,
                right: EVENT_KEY.d,
                right2: EVENT_KEY.rightArrow,
                up: EVENT_KEY.w,
                up2: EVENT_KEY.upArrow,
                down: EVENT_KEY.s,
                down2: EVENT_KEY.downArrow
            }
        });
        SNAKES.push(snake);
    }
}
function setupFrame() {
    const lines = GRID.args.lines;
    const columns = GRID.args.columns;
    if (Options.get('frameOn')) {
        for (let line = 0; line < lines; line++) {
            const leftWall = new Wall();
            const rightWall = new Wall();
            GRID.add(leftWall, {
                column: 0,
                line: line
            });
            GRID.add(rightWall, {
                column: columns - 1,
                line: line
            });
        }
        for (let column = 1; column < columns - 1; column++) {
            const topWall = new Wall();
            const bottomWall = new Wall();
            GRID.add(topWall, {
                column: column,
                line: 0
            });
            GRID.add(bottomWall, {
                column: column,
                line: lines - 1
            });
        }
    }
}
function setupAppleInterval() {
    const appleInterval = Options.get('appleInterval');
    const interval = new Interval({
        callback: function () {
            const position = GRID.getRandomEmptyPosition();
            const food = new Apple();
            GRID.add(food, position);
        },
        interval: appleInterval
    });
    INTERVALS.push(interval);
}
function setupOrangeInterval() {
    const orangeInterval = Options.get('orangeInterval');
    const interval = new Interval({
        callback: function () {
            const position = GRID.getRandomEmptyPosition();
            const food = new Orange();
            GRID.add(food, position);
        },
        interval: orangeInterval
    });
    INTERVALS.push(interval);
}
function setupBananaInterval() {
    const bananaInterval = Options.get('bananaInterval');
    const interval = new Interval({
        callback: function () {
            const position = GRID.getRandomEmptyPosition();
            const food = new Banana();
            GRID.add(food, position);
        },
        interval: bananaInterval
    });
    INTERVALS.push(interval);
}
function setupSnakeMovement() {
    for (let a = 0; a < SNAKES.length; a++) {
        const snake = SNAKES[a];
        const snakeInterval = () => {
            return snake.getMovementInterval();
        };
        const interval = new Interval({
            callback: () => {
                snake.movementTick();
                const tails = snake.getAllTails();
                for (let b = 0; b < tails.length; b++) {
                    const tail = tails[b];
                    tail.tick();
                    const next = tail.nextPosition();
                    GRID.move(tail, next);
                }
            },
            interval: snakeInterval
        });
        INTERVALS.push(interval);
    }
}
function updateScore(snake) {
    GameMenu.updateScore(SNAKES.indexOf(snake), snake.getAllTails().length);
}
function tailFoodCollision(tail, food) {
    const color = tail.color;
    const position = SNAKES_COLORS.indexOf(color);
    if (position < 0) {
        throw Error('Invalid tail color.');
    }
    const snake = SNAKES[position];
    snake.eat(food);
    GRID.remove(food);
    updateScore(snake);
}
function tailTailCollision(tail1, tail2) {
    tail1.asBeenHit();
    tail2.asBeenHit();
    over();
}
function tailWallCollision(tail, wall) {
    tail.asBeenHit();
    wall.asBeenHit();
    over();
}
function dealWithCollision(items) {
    if (GAME_OVER) {
        return;
    }
    const a = items.a;
    const b = items.b;
    const typeA = a.type;
    const typeB = b.type;
    if (typeA === ItemType.tail && typeB === ItemType.food) {
        tailFoodCollision(a, b);
    }
    else if (typeA === ItemType.food && typeB === ItemType.tail) {
        tailFoodCollision(b, a);
    }
    else if (typeA === ItemType.tail && typeB === ItemType.tail) {
        tailTailCollision(a, b);
    }
    else if (typeA === ItemType.tail && typeB === ItemType.wall) {
        tailWallCollision(a, b);
    }
    else if (typeA === ItemType.wall && typeB === ItemType.tail) {
        tailWallCollision(b, a);
    }
}
function collisionOccurred(items) {
    COLLISIONS.push(items);
}
export function over() {
    if (GAME_OVER) {
        return;
    }
    GAME_OVER = true;
    let text = 'Game Over!<br />';
    let score = 0;
    if (TWO_PLAYER_MODE) {
        var player1_score = SNAKES[0].getNumberOfTails();
        var player2_score = SNAKES[1].getNumberOfTails();
        if (player1_score > player2_score) {
            text += 'Player 1 Won!<br />';
            score = player1_score;
        }
        else if (player2_score > player1_score) {
            text += 'Player 2 Won!<br />';
            score = player2_score;
        }
        else {
            text += 'Draw!<br />';
            score = player1_score;
        }
    }
    else {
        score = SNAKES[0].getNumberOfTails();
    }
    text += `Score: <span class="displayValue">${score}</span>`;
    pause();
    addScores();
    const popup = new PopupWindow({
        content: text,
        buttons: [
            {
                text: 'Restart',
                onClick: function () {
                    popup.remove();
                    clear();
                    start(MAP_NAME, TWO_PLAYER_MODE);
                }
            },
            {
                text: 'Quit',
                onClick: function () {
                    popup.remove();
                    quit(false);
                }
            }
        ]
    });
}
;
function addScores() {
    TIMER.stop();
    for (var i = 0; i < SNAKES.length; i++) {
        HighScore.add(MAP_NAME, SNAKES[i].getNumberOfTails(), TIMER.getMilliseconds());
    }
    HighScore.save();
}
export function quit(saveScores = true) {
    if (saveScores) {
        addScores();
    }
    clear();
    showHideCanvas(false);
    MainMenu.open('mainMenu');
}
function pause() {
    PAUSE = true;
}
function resume() {
    PAUSE = false;
}
export function isPaused() {
    return PAUSE;
}
export function clear() {
    TIMEOUTS.length = 0;
    INTERVALS.length = 0;
    COLLISIONS.length = 0;
    SNAKES.length = 0;
    TIMER.stop();
    GRID.removeAll();
    GameMenu.clear();
    resume();
}
export function pauseResume(pauseGame) {
    if (pauseGame) {
        TIMER.stop();
        pause();
    }
    else {
        TIMER.start();
        resume();
    }
}
export function isTwoPlayersMode() {
    return TWO_PLAYER_MODE;
}
export function isGameOver() {
    return GAME_OVER;
}
export function addToStage(displayObject) {
    STAGE.addChild(displayObject);
}
export function removeFromStage(displayObject) {
    STAGE.removeChild(displayObject);
}
export function addToGrid(element, desiredPosition) {
    const position = GRID.getValidPosition(desiredPosition);
    GRID.add(element, position);
}
export function addTimeout(args) {
    const timeout = new Timeout(args);
    TIMEOUTS.push(timeout);
}
function tick(event) {
    if (PAUSE) {
        return;
    }
    const delta = event.delta;
    for (let a = TIMEOUTS.length - 1; a >= 0; a--) {
        const timeout = TIMEOUTS[a];
        const over = timeout.tick(delta);
        if (over) {
            TIMEOUTS.splice(a, 1);
        }
    }
    for (let a = 0; a < INTERVALS.length; a++) {
        const interval = INTERVALS[a];
        interval.tick(delta);
    }
    for (let a = 0; a < COLLISIONS.length; a++) {
        const items = COLLISIONS[a];
        dealWithCollision(items);
    }
    COLLISIONS.length = 0;
    TIMER.tick(delta);
    STAGE.update();
}
