import Tail from "./tail.js";
import { addToGrid, addTimeout } from './game.js';
import { Direction } from '../main.js';
export default class Snake {
    constructor(args) {
        this.speed = 0;
        this.interval = 0;
        this.all_tails = [];
        this.color = args.color;
        this.setSpeed(args.speed);
        this.keys_held = {
            left: false,
            right: false,
            up: false,
            down: false
        };
        this.keyboard_mapping = args.keyboardMapping;
        this.first_tail = this.addTail(args.position, args.startingDirection);
    }
    addTail(position, direction) {
        const last = this.getLastTail();
        if (typeof position === 'undefined') {
            const lastDirection = last.getCurrentDirection();
            const lastPosition = last.position;
            if (lastDirection === Direction.west) {
                position = {
                    column: lastPosition.column + 1,
                    line: lastPosition.line
                };
            }
            else if (lastDirection === Direction.east) {
                position = {
                    column: lastPosition.column - 1,
                    line: lastPosition.line
                };
            }
            else if (lastDirection === Direction.north) {
                position = {
                    column: lastPosition.column,
                    line: lastPosition.line + 1
                };
            }
            else if (lastDirection === Direction.south) {
                position = {
                    column: lastPosition.column,
                    line: lastPosition.line - 1
                };
            }
            else {
                throw Error("Invalid direction on the last tail.");
            }
        }
        if (typeof direction === 'undefined') {
            direction = last.getCurrentDirection();
        }
        let path = [];
        if (this.all_tails.length !== 0) {
            path = last.clonePath();
        }
        var tail = new Tail(this.color, direction, path);
        this.all_tails.push(tail);
        addToGrid(tail, position);
        return tail;
    }
    changeDirection(newDirection) {
        var currentDirection = this.getDirection();
        if (currentDirection == newDirection) {
            return;
        }
        if ((currentDirection == Direction.west && newDirection == Direction.east) ||
            (currentDirection == Direction.east && newDirection == Direction.west) ||
            (currentDirection == Direction.north && newDirection == Direction.south) ||
            (currentDirection == Direction.south && newDirection == Direction.north)) {
            return;
        }
        const position = this.first_tail.position;
        const numberOfTails = this.all_tails.length;
        for (var i = 0; i < numberOfTails; i++) {
            const tail = this.all_tails[i];
            tail.addNewDirection(Object.assign({}, position, { direction: newDirection }));
        }
    }
    getNumberOfTails() {
        return this.all_tails.length;
    }
    getTail(position) {
        if (position < 0 || position >= this.all_tails.length) {
            return null;
        }
        return this.all_tails[position];
    }
    getLastTail() {
        return this.all_tails[this.all_tails.length - 1];
    }
    getDirection() {
        return this.first_tail.getCurrentDirection();
    }
    setSpeed(speed) {
        this.speed = speed;
        this.interval = 1 / this.speed * 1000;
    }
    getMovementInterval() {
        return this.interval;
    }
    onKeyDown(keyCode) {
        var keysHeld = this.keys_held;
        var keyboardMapping = this.keyboard_mapping;
        switch (keyCode) {
            case keyboardMapping.left:
            case keyboardMapping.left2:
                keysHeld.left = true;
                return false;
            case keyboardMapping.right:
            case keyboardMapping.right2:
                keysHeld.right = true;
                return false;
            case keyboardMapping.up:
            case keyboardMapping.up2:
                keysHeld.up = true;
                return false;
            case keyboardMapping.down:
            case keyboardMapping.down2:
                keysHeld.down = true;
                return false;
        }
        return true;
    }
    onKeyUp(keyCode) {
        var keysHeld = this.keys_held;
        var keyboardMapping = this.keyboard_mapping;
        switch (keyCode) {
            case keyboardMapping.left:
            case keyboardMapping.left2:
                keysHeld.left = false;
                return false;
            case keyboardMapping.right:
            case keyboardMapping.right2:
                keysHeld.right = false;
                return false;
            case keyboardMapping.up:
            case keyboardMapping.up2:
                keysHeld.up = false;
                return false;
            case keyboardMapping.down:
            case keyboardMapping.down2:
                keysHeld.down = false;
                return false;
        }
        return true;
    }
    eat(food) {
        const effects = food.eaten;
        for (let a = 0; a < effects.tails; a++) {
            this.addTail();
        }
        const speed = effects.speed;
        if (speed) {
            this.setSpeed(this.speed * speed.multiplier);
            addTimeout({
                callback: () => {
                    this.setSpeed(this.speed / speed.multiplier);
                },
                duration: speed.duration
            });
        }
    }
    movementTick() {
        var keysHeld = this.keys_held;
        var direction = this.getDirection();
        if (keysHeld.left) {
            if (keysHeld.down) {
                if (direction == Direction.west || direction == Direction.east) {
                    this.changeDirection(Direction.south);
                }
                else if (direction == Direction.south || direction == Direction.north) {
                    this.changeDirection(Direction.west);
                }
            }
            else if (keysHeld.up) {
                if (direction == Direction.west || direction == Direction.east) {
                    this.changeDirection(Direction.north);
                }
                else if (direction == Direction.north || direction == Direction.south) {
                    this.changeDirection(Direction.west);
                }
            }
            else {
                this.changeDirection(Direction.west);
            }
        }
        else if (keysHeld.right) {
            if (keysHeld.down) {
                if (direction == Direction.east || direction == Direction.west) {
                    this.changeDirection(Direction.south);
                }
                else if (direction == Direction.south || direction == Direction.north) {
                    this.changeDirection(Direction.east);
                }
            }
            else if (keysHeld.up) {
                if (direction == Direction.east || direction == Direction.west) {
                    this.changeDirection(Direction.north);
                }
                else if (direction == Direction.north || direction == Direction.south) {
                    this.changeDirection(Direction.east);
                }
            }
            else {
                this.changeDirection(Direction.east);
            }
        }
        else if (keysHeld.up) {
            this.changeDirection(Direction.north);
        }
        else if (keysHeld.down) {
            this.changeDirection(Direction.south);
        }
    }
    getAllTails() {
        return [...this.all_tails];
    }
}
