import { Direction } from "../main.js";
import { Grid, ItemType } from "./grid.js";
import { addToStage, removeFromStage } from './game.js';
export default class Tail {
    constructor(color, direction, path) {
        this.type = ItemType.tail;
        this.color = color;
        this.path = path;
        this.direction = direction;
        this.position = {
            column: 0,
            line: 0
        };
        this.shape = this.draw();
    }
    draw() {
        var snakeTail = new createjs.Shape();
        snakeTail.regX = Grid.halfSize;
        snakeTail.regY = Grid.halfSize;
        var g = snakeTail.graphics;
        g.beginFill(this.color);
        g.drawRoundRect(0, 0, Grid.size, Grid.size, 2);
        addToStage(snakeTail);
        return snakeTail;
    }
    asBeenHit() {
        var g = this.shape.graphics;
        g.beginFill('red');
        g.drawRoundRect(0, 0, Grid.size, Grid.size, 2);
    }
    remove() {
        removeFromStage(this.shape);
    }
    addNewDirection(path) {
        this.path.push(path);
    }
    nextPosition() {
        const current = this.position;
        switch (this.direction) {
            case Direction.west:
                return {
                    column: current.column - 1,
                    line: current.line
                };
            case Direction.east:
                return {
                    column: current.column + 1,
                    line: current.line
                };
            case Direction.north:
                return {
                    column: current.column,
                    line: current.line - 1
                };
            case Direction.south:
                return {
                    column: current.column,
                    line: current.line + 1
                };
            default:
                throw Error("Invalid 'tail' direction.");
        }
    }
    clonePath() {
        return JSON.parse(JSON.stringify(this.path));
    }
    getCurrentDirection() {
        return this.direction;
    }
    tick() {
        var direction = this.direction;
        if (this.path.length !== 0) {
            const checkpoint = this.path[0];
            const current = this.position;
            if (checkpoint.column === current.column &&
                checkpoint.line === current.line) {
                direction = checkpoint.direction;
                this.direction = direction;
                this.path.splice(0, 1);
            }
        }
    }
}
