import * as AppStorage from './storage/app_storage.js';
import * as Options from './storage/options.js';
import * as MainMenu from './menu/main_menu.js';
import * as HighScore from './storage/high_score.js';
import * as GameMenu from './game/game_menu.js';
import * as Game from './game/game.js';
import * as Preload from './other/preload.js';
import * as Message from "./other/message.js";
import { Grid } from "./game/grid.js";
export var Direction;
(function (Direction) {
    Direction[Direction["west"] = 0] = "west";
    Direction[Direction["east"] = 1] = "east";
    Direction[Direction["north"] = 2] = "north";
    Direction[Direction["south"] = 3] = "south";
    Direction[Direction["northWest"] = 4] = "northWest";
    Direction[Direction["northEast"] = 5] = "northEast";
    Direction[Direction["southWest"] = 6] = "southWest";
    Direction[Direction["southEast"] = 7] = "southEast";
})(Direction || (Direction = {}));
let CANVAS;
window.onload = function () {
    Message.init();
    Message.show('Loading...');
    AppStorage.getData(['snake_high_score', 'snake_options', 'snake_has_run_before', 'snake_selected_map'], function (data) {
        initApp(data);
        Message.hide();
        showHideCanvas(false);
    });
};
function initApp(data) {
    Options.load(data['snake_options']);
    CANVAS = document.querySelector('#MainCanvas');
    updateCanvasDimensions();
    createjs.Ticker.timingMode = createjs.Ticker.RAF;
    HighScore.load(data['snake_high_score']);
    MainMenu.init(data['snake_selected_map']);
    GameMenu.init();
    Game.init(CANVAS);
    var callback;
    if (!data['snake_has_run_before']) {
        AppStorage.setData({ snake_has_run_before: true });
        callback = () => { MainMenu.open('help'); };
    }
    else {
        callback = () => { MainMenu.open('mainMenu'); };
    }
    Preload.init(callback);
}
export function updateCanvasDimensions() {
    const columns = Options.get('columns');
    const lines = Options.get('lines');
    CANVAS.width = columns * Grid.size;
    CANVAS.height = lines * Grid.size;
}
export function showHideCanvas(show) {
    if (CANVAS) {
        if (show) {
            CANVAS.classList.remove('hidden');
        }
        else {
            CANVAS.classList.add('hidden');
        }
    }
}
