import { getMapScores } from '../storage/high_score.js';
import { joinAndCapitalize, splitCamelCaseWords, boolToOnOff, timeToString } from '../other/utilities.js';
import { open } from './main_menu.js';
let CURRENT_SCORE;
let CURRENT_BUTTON;
export function initHighScore() {
    var back = document.getElementById('HighScore-Back');
    back.onclick = function () {
        hideInfoWindow();
        open('mainMenu');
    };
}
export function buildHighScoreTable(mapName) {
    const title = document.getElementById('HighScore-Title');
    const table = document.getElementById('HighScore-Table');
    const displayName = joinAndCapitalize(splitCamelCaseWords(mapName));
    table.innerHTML = '';
    title.innerHTML = `High score (${displayName})`;
    var allScores = getMapScores(mapName);
    if (!allScores || allScores.length === 0) {
        table.innerHTML = 'No score yet.';
    }
    else {
        var tableRow = document.createElement('tr');
        var header = ['Position', 'Number Of Tails', 'Time'];
        var tableHeader;
        for (var i = 0; i < header.length; i++) {
            tableHeader = document.createElement('th');
            tableHeader.innerText = header[i];
            tableRow.appendChild(tableHeader);
        }
        table.appendChild(tableRow);
        for (i = 0; i < allScores.length; i++) {
            const score = allScores[i];
            const tableRow = document.createElement('tr');
            const position = document.createElement('td');
            const numberOfTails = document.createElement('td');
            const time = document.createElement('td');
            const info = document.createElement('td');
            info.innerText = 'Info';
            info.className = 'button';
            position.innerText = (i + 1).toString();
            numberOfTails.innerText = score.numberOfTails.toString();
            time.innerText = timeToString(score.time) + 's';
            info.onclick = function () {
                showInfoWindow(score, info);
            };
            tableRow.appendChild(position);
            tableRow.appendChild(numberOfTails);
            tableRow.appendChild(time);
            tableRow.append(info);
            table.appendChild(tableRow);
        }
    }
}
function showInfoWindow(score, button) {
    const container = document.getElementById('HighScore-Info');
    const options = score.options;
    if (CURRENT_SCORE === score) {
        hideInfoWindow();
        return;
    }
    clearCurrent();
    CURRENT_SCORE = score;
    CURRENT_BUTTON = button;
    container.innerHTML = '';
    const body = document.createElement('div');
    const close = document.createElement('div');
    addInfoValue(body, 'Tails: ', score.numberOfTails);
    addInfoValue(body, 'Time: ', timeToString(score.time), 's');
    addInfoValue(body, 'Columns: ', options.columns);
    addInfoValue(body, 'Lines: ', options.lines);
    addInfoValue(body, 'Frame: ', boolToOnOff(options.frameOn));
    addInfoValue(body, 'Apple interval: ', options.appleInterval, 'ms');
    addInfoValue(body, 'Orange interval: ', options.orangeInterval, 'ms');
    addInfoValue(body, 'Banana interval: ', options.bananaInterval, 'ms');
    addInfoValue(body, 'Wall interval: ', options.wallInterval, 'ms');
    addInfoValue(body, 'Snake speed: ', options.snakeSpeed, 'Hz');
    close.className = 'button backButton';
    close.innerText = 'Close';
    close.onclick = () => {
        hideInfoWindow();
    };
    container.appendChild(body);
    container.appendChild(close);
    container.classList.remove('hidden');
    button.classList.add('HighScore-selectedInfo');
}
function hideInfoWindow() {
    const container = document.getElementById('HighScore-Info');
    container.classList.add('hidden');
    clearCurrent();
}
function clearCurrent() {
    if (CURRENT_BUTTON) {
        CURRENT_BUTTON.classList.remove('HighScore-selectedInfo');
    }
    CURRENT_BUTTON = undefined;
    CURRENT_SCORE = undefined;
}
function addInfoValue(container, text, value, unit) {
    const textElement = document.createElement('div');
    textElement.innerText = text;
    const valueElement = document.createElement('span');
    valueElement.className = 'displayValue';
    valueElement.innerText = value;
    const unitElement = document.createElement('span');
    if (unit) {
        unitElement.innerText = ` ${unit}`;
    }
    textElement.appendChild(valueElement);
    textElement.appendChild(unitElement);
    container.append(textElement);
}
