import * as AppStorage from '../storage/app_storage.js';
import * as Game from '../game/game.js';
import { buildHighScoreTable, initHighScore } from "./high_score_menu.js";
import { initOptions } from "./options_menu.js";
var MAP_SELECTED;
let PAGES;
export function init(mapName) {
    PAGES = {
        mainMenu: document.getElementById('MainMenu'),
        options: document.getElementById('Options'),
        highScore: document.getElementById('HighScore'),
        help: document.getElementById('Help')
    };
    initMainMenu(mapName);
    initOptions();
    initHighScore();
    initHelp();
}
export function open(page) {
    const container = PAGES[page];
    clear();
    container.classList.remove('hidden');
}
function changeMap(element, save = true) {
    if (MAP_SELECTED) {
        MAP_SELECTED.classList.remove('MainMenu-mapSelected');
    }
    if (save !== false) {
        const selected = element.getAttribute('data-map');
        AppStorage.setData({ 'snake_selected_map': selected || undefined });
    }
    element.classList.add('MainMenu-mapSelected');
    MAP_SELECTED = element;
}
function initMainMenu(mapName) {
    var startGame = document.getElementById('MainMenu-StartGame');
    var startGame_2players = document.getElementById('MainMenu-StartGame-2players');
    var selectMap = document.getElementById('MainMenu-SelectMap');
    var options = document.getElementById('MainMenu-Options');
    var highScore = document.getElementById('MainMenu-HighScore');
    var help = document.getElementById('MainMenu-Help');
    startGame.onclick = function () {
        clear();
        Game.start(MAP_SELECTED.getAttribute('data-map'), false);
    };
    startGame_2players.onclick = function () {
        clear();
        Game.start(MAP_SELECTED.getAttribute('data-map'), true);
    };
    for (let a = 0; a < selectMap.children.length; a++) {
        let map = selectMap.children[a];
        map.onclick = function (event) {
            changeMap(map);
        };
    }
    if (mapName) {
        changeMap(selectMap.querySelector('li[data-map="' + mapName + '"]'), false);
    }
    else {
        changeMap(selectMap.firstElementChild, false);
    }
    options.onclick = function () {
        open('options');
    };
    highScore.onclick = function () {
        buildHighScoreTable(MAP_SELECTED.getAttribute('data-map'));
        open('highScore');
    };
    help.onclick = function () {
        open('help');
    };
}
function initHelp() {
    var back = document.getElementById('Help-Back');
    back.onclick = function () {
        open('mainMenu');
    };
}
function clear() {
    const keys = Object.keys(PAGES);
    keys.map((key) => {
        const container = PAGES[key];
        container.classList.add('hidden');
    });
}
