import * as AppStorage from './app_storage.js';
import * as Options from './options.js';
var HIGH_SCORE = {};
var HIGH_SCORE_LENGTH = 5;
function isMapScores(object) {
    if (object) {
        const keys = Object.keys(object);
        for (let a = 0; a < keys.length; a++) {
            const mapName = keys[a];
            const mapScores = object[mapName];
            if (!Array.isArray(mapScores)) {
                return false;
            }
            for (let b = 0; b < mapScores.length; b++) {
                const score = mapScores[b];
                if (!(score instanceof Object) ||
                    !score.hasOwnProperty('numberOfTails') ||
                    !score.hasOwnProperty('time') ||
                    !score.hasOwnProperty('options')) {
                    return false;
                }
            }
        }
        return true;
    }
    return false;
}
export function load(score) {
    if (isMapScores(score)) {
        HIGH_SCORE = score;
    }
}
export function save() {
    AppStorage.setData({ snake_high_score: HIGH_SCORE });
}
export function add(mapName, numberOfTails, time) {
    if (numberOfTails <= 1) {
        return;
    }
    let scoreArray = HIGH_SCORE[mapName];
    const options = Options.clone();
    const intTime = Math.round(time);
    if (typeof scoreArray === 'undefined') {
        scoreArray = [];
        HIGH_SCORE[mapName] = scoreArray;
    }
    scoreArray.push({
        numberOfTails: numberOfTails,
        time: intTime,
        options: options
    });
    scoreArray.sort(function (a, b) {
        return b.numberOfTails - a.numberOfTails;
    });
    if (scoreArray.length > HIGH_SCORE_LENGTH) {
        scoreArray.pop();
    }
}
export function getMapScores(mapName) {
    return HIGH_SCORE[mapName];
}
